#include "main.h"

/* private typedef */
/* private functions */
static void GPIO_Init(void);
static void I2C1_Init(void);
static void SPI1_Init(void);
static void TIM1_Init(void);
//static void TIM3_Init(void);
//static void TIM14_Init(void);
//static void TIM16_Init(void);
static void TIM17_Init(void);
//static void USART1_UART_Init(void);

/* Board perephireal Configuration  */
void Board_Init(void)
{
  /* At this stage the microcontroller clock setting is already configured, 
     this is done through SystemInit() function which is called from startup
     file (startup_stm32f072xb.s) before to branch to application main.
     To reconfigure the default setting of SystemInit() function, refer to
     system_stm32f0xx.c file
  */

  /* Main peripheral clock enable */
  RCC->AHBENR |= (RCC_AHBENR_GPIOAEN | RCC_AHBENR_GPIOBEN | RCC_AHBENR_DMAEN);
  RCC->APB1ENR = (RCC_APB1ENR_PWREN | RCC_APB1ENR_I2C1EN); // | RCC_APB1ENR_TIM14EN| RCC_APB1ENR_TIM3EN);
  RCC->APB2ENR = (RCC_APB2ENR_SYSCFGEN | RCC_APB2ENR_SPI1EN | RCC_APB2ENR_TIM1EN | RCC_APB2ENR_TIM17EN); // | RCC_APB2ENR_TIM16EN

  /* Peripheral interrupt init*/
  /* RCC_IRQn interrupt configuration */
  NVIC_SetPriority(RCC_IRQn, 0);
  NVIC_EnableIRQ(RCC_IRQn);

  /* Initialize all configured peripherals */
  //HT1632C_CS_OFF;
  //HT1632C_WR_HGH;
  //HT1632C_DATA_1;
  GPIO_Init();

  /* DMA interrupt init */
  /* DMA1_Channel2_3_IRQn interrupt configuration */
  NVIC_SetPriority(DMA1_Channel2_3_IRQn, 0);
  NVIC_EnableIRQ(DMA1_Channel2_3_IRQn);

  I2C1_Init();

  SPI1_Init();
  /** Star SPI transfer to shift registers */
  /* DMA Destination addr: SPI1 data register. */
  DMA1_Channel3->CPAR = (uint32_t)&(SPI1->DR);
  /* Enable SPI transfer */
  SPI1->CR1 |= SPI_CR1_SPE;
//  Flag.SPI_TX_End = 1;

  TIM1_Init();
//  TIM3_Init();
//  TIM14_Init();
//  TIM16_Init();
  TIM17_Init();
//  USART1_UART_Init();
}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void GPIO_Init(void)
{
  /* EXTI Line: falling, pull-up, input */
  SYSCFG->EXTICR[3] = 0;
  /* Enable IT on provided Lines */
  EXTI->IMR |= EXTI_IMR_IM12;
  /* Enable Falling Trigger on provided Lines */
  EXTI->FTSR |= EXTI_IMR_IM12;
  /* EXTI interrupt init*/
  NVIC_SetPriority(EXTI4_15_IRQn, 0);
  NVIC_EnableIRQ(EXTI4_15_IRQn);

  /* Select output mode (01) PP+PU, High Speed
   PA8  - Buzzer (AF2 for TIM1_CH1), Pull-Down
   PA15 - ~CS / SPI_NSS (AF0)
   * Select output mode (10) AF+PP, High Speed
   PB3  - ~WR / SPI_SCK (AF0)
   PB5  - Data / SPI_MOSI (AF0)
   * Select output mode (10) AF+OD, High Speed, Pull-Up
   PB6  - SCL (AF1)
   PB7  - SDA (AF1)
  */
  // MODE Output
  GPIOA->MODER = (GPIOA->MODER & ~(GPIO_MODER_MODER8 | GPIO_MODER_MODER15)) \
                | (GPIO_MODER_MODER8_1 | GPIO_MODER_MODER15_0);
  GPIOB->MODER = (GPIOB->MODER & ~(GPIO_MODER_MODER3|GPIO_MODER_MODER5|GPIO_MODER_MODER6|GPIO_MODER_MODER7)) \
                | (GPIO_MODER_MODER3_1|GPIO_MODER_MODER5_1|GPIO_MODER_MODER6_1|GPIO_MODER_MODER7_1);
  // Pull-Up
  GPIOA->PUPDR = (GPIOA->PUPDR & ~(GPIO_PUPDR_PUPDR8 | GPIO_PUPDR_PUPDR15)) \
                | (GPIO_PUPDR_PUPDR8_1 | GPIO_PUPDR_PUPDR15_0);
  GPIOB->PUPDR = (GPIOA->PUPDR & ~(GPIO_PUPDR_PUPDR6 | GPIO_PUPDR_PUPDR7)) \
                | (GPIO_PUPDR_PUPDR6_0 | GPIO_PUPDR_PUPDR7_0);
  // High Speed
  GPIOA->OSPEEDR = (GPIO_OSPEEDR_OSPEEDR8|GPIO_OSPEEDR_OSPEEDR15);
  GPIOB->OSPEEDR = (GPIO_OSPEEDR_OSPEEDR3|GPIO_OSPEEDR_OSPEEDR5 \
                  |GPIO_OSPEEDR_OSPEEDR6|GPIO_OSPEEDR_OSPEEDR7);
  // Open Drain
  GPIOB->OTYPER = (GPIO_OTYPER_OT_6|GPIO_OTYPER_OT_7);
  // AF2 for PA8
  GPIOA->AFR[1] = 0x2;
  // AF1 for PB6 & PB7
  GPIOB->AFR[0] = (0x1<<24) | (0x1<<28);

  /* Select Pull-Up for input pins
   PA0 - BTN1 / H+
   PA1 - BTN2 / M+
   PA2 - SW1 / Stop
   PA3 - SW2 / Sec
   PA4 - BTN3 / Res
   PA5 - SW3 / AlarmSet
   PA6 - SW5 / AlarmOff
   PA7 - SW4 / Bright
   PA12 - RTC IRQ / Exti
  */
  GPIOA->PUPDR |= (GPIO_PUPDR_PUPDR0_0|GPIO_PUPDR_PUPDR1_0|GPIO_PUPDR_PUPDR2_0 \
                |GPIO_PUPDR_PUPDR3_0|GPIO_PUPDR_PUPDR4_0|GPIO_PUPDR_PUPDR5_0 \
                |GPIO_PUPDR_PUPDR6_0|GPIO_PUPDR_PUPDR7_0);
}

/**
  * @brief Configure SPI pin for software mode
  */
void GPIO_SPI_SW(void) {
  // GPIO pin mode
  GPIOB->MODER &= ~(GPIO_MODER_MODER3 | GPIO_MODER_MODER5);
  GPIOB->MODER |= (GPIO_MODER_MODER3_0 | GPIO_MODER_MODER5_0);
}

/**
  * @brief Configure SPI pin for hardware mode
  */
void GPIO_SPI_HW(void) {
  // SPI pin mode
  GPIOB->MODER &= ~(GPIO_MODER_MODER3 | GPIO_MODER_MODER5);
  GPIOB->MODER |= (GPIO_MODER_MODER3_1 | GPIO_MODER_MODER5_1);
}

/**
  * @brief I2C1 Initialization Function
  * @param None
  * @retval None
  */
static void I2C1_Init(void)
{
  /* By default, I2C clocked from HSI, for SysClk use this: */
  //RCC->CFGR3 |= RCC_CFGR3_I2C1SW;

  /** I2C Initialization: I2C_Fast */
  I2C1->CR1 = 0x0;
  I2C1->TIMINGR = 0x0010020A; //0x00701850; // 400kHz / 8MHz / 100/100 ns
  I2C1->CR2 = I2C_CR2_AUTOEND;
  I2C1->CR1 |= I2C_CR1_PE;
}

/**
  * @brief SPI1 Initialization Function
  * @param None
  * @retval None
  */
static void SPI1_Init(void)
{
  /* SPI1 DMA Init */
  /* SPI1_TX Init: Priority high, Memory increment, read from memory, non-circular mode,
     Enable DMA transfer complete/error interrupts */
  DMA1_Channel3->CCR = (DMA_CCR_PL_1 | DMA_CCR_MINC | DMA_CCR_DIR | DMA_CCR_CIRC | DMA_CCR_TCIE); // | DMA_CCR_TEIE

  /* SPI1 interrupt Init */
  NVIC_SetPriority(SPI1_IRQn, 0);
  NVIC_EnableIRQ(SPI1_IRQn);

  /* SPI1 parameter configuration: master mode, data 8 bit, divider = 2, TX DMA SPI_CR1_BR_2 | SPI_CR1_BR_0 |  */
  SPI1->CR1 = (SPI_CR1_MSTR | SPI_CR1_SSM | SPI_CR1_SSI | SPI_CR1_CPOL | SPI_CR1_CPHA | SPI_CR1_LSBFIRST);
  SPI1->CR2 = (SPI_CR2_DS_2 | SPI_CR2_DS_1 | SPI_CR2_DS_0 | SPI_CR2_TXDMAEN);
}

/**
  * @brief TIM1 Initialization Function
  * @param None
  * @retval None
  */
static void TIM1_Init(void)
{
  /* target clock */
  TIM1->PSC = 47; // 1 MHz counter
  TIM1->ARR = 1000; // 1 KHz, auto reload value
  TIM1->CR1 = TIM_CR1_ARPE;
  // initial pwm value
  TIM1->CCR1 = 500; // PWM 50% - max volume
  // pwm mode 1 for 4 chanels
  TIM1->CCMR1 = (TIM_CCMR1_OC1M_2 | TIM_CCMR1_OC1M_1 | TIM_CCMR1_OC1PE);
  // reset int flag - not needed, int unused
  //TIM1->SR |= TIM_SR_UIF;
  TIM1->BDTR = TIM_BDTR_MOE; // enable main output
  TIM1->EGR = TIM_EGR_UG; // force timer update
  /* TIM1 CC_EnableChannel */
  TIM1->CCER = TIM_CCER_CC1E;
}

/**
  * @brief TIM17 Initialization Function
  * @param None
  * @retval None
  */
static void TIM17_Init(void)
{
  /* setup clock */
  TIM17->PSC = 47; // prescaler for 1uS
  TIM17->ARR = 1000; // auto reload value for 1 mS
  TIM17->CR1 = (TIM_CR1_ARPE | TIM_CR1_OPM); // one pulse mode
  TIM17->BDTR = TIM_BDTR_MOE; // enable main output
  TIM17->EGR = TIM_EGR_UG; // force timer update
}

/**
 * @brief Buzzer control: On.
 */
void buzer_On(void) {
  BUZER_ON;
}

/**
 * @brief Buzzer control: Off.
 */
void buzer_Off(void) {
  BUZER_OFF;
}

/**
  * @brief This function handles EXTI line 4 to 15 interrupts.
  */
void EXTI4_15_IRQHandler(void)
{
  if ((EXTI->PR & EXTI_IMR_IM12) != 0)
  {
    EXTI->PR = EXTI_IMR_IM12;
    Flag.RTC_IRQ = 1;
    //ES_PlaceEvent(evNewSecond);
  }
}

/**
  * @brief This function handles DMA1 channel 2 and 3 interrupts.
  */
void DMA1_Channel2_3_IRQHandler(void) {
  /* SPI Transmit Channel */
  if ((DMA1->ISR & DMA_ISR_TCIF3) != 0) {
    // clear interrupt flag
    DMA1->IFCR = DMA_IFCR_CTCIF3;
    // disable transfer
    SPI1->CR2 &= ~(SPI_CR2_TXDMAEN);
    DMA1_Channel3->CCR &= ~(DMA_CCR_EN);
    // wite for spi transaction
    while ((SPI1->SR & SPI_SR_BSY) != 0) { __NOP(); }
    // deselect chip
    HT1632C_CS_OFF;
  }

  /* I2C Transmit Channel */
  if ((DMA1->ISR & DMA_ISR_TCIF2) != 0) {
    // clear interrupt flag
    DMA1->IFCR = DMA_IFCR_CTCIF2;
    // Disable DMA channels for I2C TX
    DMA1_Channel2->CCR &= ~(DMA_CCR_EN);
    Flag.I2C_TX_End = 1;
  }
  // I2C Transmit Error
  if ((DMA1->ISR & DMA_ISR_TEIF2) != 0) {
    DMA1->IFCR = DMA_IFCR_CTEIF2;
    DMA1_Channel2->CCR &= ~(DMA_CCR_EN);
    Flag.I2C_TX_End = 1;
    Flag.I2C_TX_Err = 1;
  }
}

/**
  * @brief This function handles I2C1 global interrupt.
  */
void I2C1_IRQHandler(void) {
  if ((I2C1->ISR & I2C_ISR_RXNE) != 0) {
    I2C1->RXDR;
  }
}
